using System.ComponentModel;
using System.Windows.Input;
using Telerik.Windows.Data;

namespace BindingToLinqToSql.ViewModels
{
    public class ProductsViewModel : INotifyPropertyChanged
    {
        private readonly NorthwindDataContext context;
        private readonly DelegateCommand<Product> addProductCommand;
        private readonly DelegateCommand<Product> deleteProductCommand;
        private readonly QueryableCollectionView productsCollectionView;
        private Product selectedProduct;

        public ProductsViewModel()
        {
            this.context = new NorthwindDataContext();
            this.addProductCommand = new DelegateCommand<Product>(this.AddProduct);
            this.deleteProductCommand = new DelegateCommand<Product>(this.DeleteProduct, this.CanDeleteProduct);

            this.productsCollectionView = new QueryableCollectionView(this.context.Products);
        }

        public event PropertyChangedEventHandler PropertyChanged;

        public ICommand AddProductCommand
        {
            get
            {
                return this.addProductCommand;
            }
        }

        public ICommand DeleteProductCommand
        {
            get
            {
                return this.deleteProductCommand;
            }
        }

        public QueryableCollectionView Products
        {
            get
            {
                return this.productsCollectionView;
            }
        }

        public Product SelectedProduct
        {
            get
            {
                return this.selectedProduct;
            }
            set
            {
                this.selectedProduct = value;
                this.OnPropertyChanged("SelectedProduct");
                this.deleteProductCommand.RaiseCanExecuteChanged();
            }
        }

        protected virtual void OnPropertyChanged(PropertyChangedEventArgs args)
        {
            if (this.PropertyChanged != null)
            {
                this.PropertyChanged(this, args);
            }
        }

        protected void OnPropertyChanged(string propertyName)
        {
            this.OnPropertyChanged(new PropertyChangedEventArgs(propertyName));
        }

        private void AddProduct(Product product)
        {
            var newProduct = product ?? DefaultProduct;
            this.InsertAndSubmitProduct(newProduct);

            this.Products.Refresh();
        }

        private static Product DefaultProduct
        {
            get
            {
                return new Product { ProductName = "Default Product" };
            }
        }

        private void InsertAndSubmitProduct(Product product)
        {
            this.context.Products.InsertOnSubmit(product);
            this.context.SubmitChanges();
        }

        private void DeleteProduct(Product p)
        {
            var deletedProduct = p ?? this.SelectedProduct;
            this.DeleteAndSubmitProduct(deletedProduct);

            this.Products.Refresh();
        }

        private void DeleteAndSubmitProduct(Product product)
        {
            this.context.Products.DeleteOnSubmit(product);
            this.context.SubmitChanges();
        }

        private bool CanDeleteProduct(Product product)
        {
            if (product != null)
            {
                return true;
            }

            return this.SelectedProduct != null;
        }
    }
}